@@ -358,6 +358,18 @@ def spawn(platform, count, role, group_name, provider=Providers.AWS, region=None
 
     return 0
 
+def _is_saved_group(vms_info, group_name):
+    group = vms_info[group_name]
+    return (group.get("meta", {}).get("saved") == True)
+
+def _delete_saved_group(vms_info, group_name):
+    print("Deleting saved group '{}' without terminating VMs:".format(group_name))
+    for name, vm in vms_info[group_name].items():
+        if name == "meta":
+            continue
+        print("  {}: {}@{} ({})".format(name, vm["user"], vm["public_ips"][0], vm["role"]))
+    del vms_info[group_name]
+
 def destroy(group_name=None):
     if os.path.exists(CLOUD_CONFIG_FPATH):
         creds_data = read_json(CLOUD_CONFIG_FPATH)
@@ -392,13 +404,19 @@ def destroy(group_name=None):
 
     to_destroy = []
     if group_name:
-        print("Destroying hosts in the '%s' group" % group_name)
         if not group_name.startswith("@"):
             group_name = "@" + group_name
         if group_name not in vms_info:
             print("Group '%s' not found" % group_name)
             return 1
 
+        if (_is_saved_group(vms_info, group_name)):
+            _delete_saved_group(vms_info, group_name)
+            write_json(CLOUD_STATE_FPATH, vms_info)
+            return 0
+
+        print("Destroying hosts in the '%s' group" % group_name)
+
         region = vms_info[group_name]["meta"]["region"]
         provider = vms_info[group_name]["meta"]["provider"]
         if provider == "aws":
@@ -426,6 +444,10 @@ def destroy(group_name=None):
     else:
         print("Destroying all hosts")
         for group_name in [key for key in vms_info.keys() if key.startswith("@")]:
+            if (_is_saved_group(vms_info, group_name)):
+                _delete_saved_group(vms_info, group_name)
+                continue
+
             region = vms_info[group_name]["meta"]["region"]
             provider = vms_info[group_name]["meta"]["provider"]
             if provider == "aws":
@@ -484,6 +506,145 @@ def init_cloud_config():
     print("Config file %s created, please complete the configuration in it." % CLOUD_CONFIG_FPATH)
     return 0
 
+
+def save(name, hosts, role):
+    state = read_json(CLOUD_STATE_FPATH)
+    if not state:
+        state = {}
+    if "@" + name in state:
+        print("Group '{}' already exists".format(name))
+        return 1
+    group = {"meta": {"saved": True}}
+    for index, host in enumerate(hosts):
+        split = host.split("@")
+        if len(split) != 2:
+            print("Host '{}' not accepted, must be given as user@ip-address".format(host))
+            return 1
+        user, ip = host.split("@")
+        instance = {
+            "public_ips": [ip],
+            "user": user,
+            "role": role,
+        }
+        group[name + "-" + str(index + 1)] = instance
+    state["@" + name] = group
+    write_json(CLOUD_STATE_FPATH, state)
+    return 0
+
+
+def _ansible_inventory():
+    if not os.path.exists(CLOUD_STATE_FPATH):
+        print("No saved cloud state info")
+        return 1
+
+    vms_info = read_json(CLOUD_STATE_FPATH)
+    all_lines = []
+    hub_lines = []
+    client_lines = []
+    for group_name in vms_info:
+        print("[%s]" % group_name.strip("@"))
+        for vm in (name for name in vms_info[group_name] if name != "meta"):
+            host_line = ('%s ansible_host=%s ansible_user=%s ansible_ssh_extra_args="-o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no"' %
+                         (vm, vms_info[group_name][vm]["public_ips"][0], vms_info[group_name][vm]["user"]))
+            print(host_line)
+            all_lines.append(host_line)
+            if vms_info[group_name][vm]["role"] == "hub":
+                hub_lines.append(host_line)
+            if vms_info[group_name][vm]["role"] == "client":
+                client_lines.append(host_line)
+        print()
+
+    if len(all_lines) > 0:
+        print("[all]")
+        for line in all_lines:
+            print(line)
+        print()
+
+    if len(hub_lines) > 1:
+        print("[hubs]")
+        for line in hub_lines:
+            print(line)
+        print()
+    elif len(hub_lines) == 1:
+        print("[hub]")
+        print(hub_lines[0])
+        print()
+
+    if len(client_lines) > 0:
+        print("[clients]")
+        for line in client_lines:
+            print(line)
+        print()
+
+    return 0
+
+def _flatten(items):
+    flattened = []
+    for item in items:
+        if isinstance(item, (list, tuple)):
+            flattened.extend(_flatten(item))
+        else:
+            flattened.append(str(item))
+    return flattened
+
+def _print_indented_and_wrapped(strings, indent, wrap):
+    strings = _flatten(strings)
+    lines = []
+    line = ""
+    for index, string in enumerate(strings):
+        if index < len(strings) - 1:
+            string += ", "
+
+        if line == "":
+            line = " " * indent + string
+            continue
+
+        if len(line + string) >= wrap:
+            lines.append(line)
+            line = " " * indent + string
+            continue
+
+        line += string
+    if line != "":
+        lines.append(line)
+    for line in lines:
+        print(line)
+
+def show(ansible_inventory):
+    if ansible_inventory:
+        return _ansible_inventory()
+    if not os.path.exists(CLOUD_STATE_FPATH):
+        print("No saved cloud state info")
+        return 1
+
+    vms_info = read_json(CLOUD_STATE_FPATH)
+    if not vms_info:
+        print("No hosts found")
+        return 0
+    hosts = 0
+    groups = len(vms_info)
+    for group_name in vms_info:
+        extra = ""
+        group = vms_info[group_name]
+        if "meta" in group:
+            meta = group["meta"]
+            del group["meta"]
+            if "region" in meta and "provider" in meta:
+                extra = " in {}, {}".format(meta["region"], meta["provider"])
+        print("{}: ({} host{}{})".format(group_name, len(group), "s" if len(group) > 1 else "", extra))
+        hosts += len(group)
+        for name, vm in group.items():
+            role = vm["role"]
+            keywords = [v for k,v in vm.items() if k not in ("user", "role")]
+            keywords = [role, name] + keywords
+            identifier = "{}@{}".format(vm["user"], vm["public_ips"][0])
+            print("  " + identifier)
+            _print_indented_and_wrapped(keywords, 4, 80)
+        print("\n")
+    print("Total: {} host{} in {} group{}".format(
+        hosts, "s" if hosts > 1 else "", groups, "s" if groups > 1 else ""))
+    return 0
+
 def uninstall(hosts):
     errors = 0
     for host in hosts:
